<?php

namespace Drupal\reader\Services;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\reader\ReaderBase;
use Symfony\Component\HttpFoundation\RequestStack;

class Reader extends ReaderBase {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * @var \Drupal\Core\Datetime\DateFormatInterface
   */
  protected $dateFormatter;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * ActivityPubFormAlter constructor
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The pager manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, DateFormatterInterface $date_formatter, PagerManagerInterface $pager_manager, RequestStack $request_stack, AccountInterface $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
    $this->dateFormatter = $date_formatter;
    $this->pagerManager = $pager_manager;
    $this->requestStack = $request_stack;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public function getChannels() {

    if (!$this->moduleHandler->moduleExists('aggregator') || !$this->currentUser->hasPermission('access reader')) {
      return [];
    }

    return [
      'label' => $this->t('Aggregator'),
      'channels' => [
        (object) [
          'uid' => 'aggregator',
          'unread' => 0,
          'name' => $this->t('All feeds')
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getTimeline($id, $search = NULL) {
    $items = [];
    $paging = [];

    if ($this->moduleHandler->moduleExists('aggregator') && $this->currentUser->hasPermission('access reader')) {
      /** @var \Drupal\aggregator\ItemInterface[] $posts */

      $page = $this->requestStack->getCurrentRequest()->get('after', 0);
      if ($page > 0) {
        $this->requestStack->getCurrentRequest()->query->set('page', $page);
      }

      $posts = $this->entityTypeManager->getStorage('aggregator_item')->loadAll(20);
      foreach ($posts as $post) {
        /** @var \Drupal\aggregator\FeedInterface $feed */
        $feed = $this->entityTypeManager->getStorage('aggregator_feed')->load($post->getFeedId());
        $item = [];
        $item['_id'] = $post->id();
        $item['_is_read'] = 1;
        $item['type'] = 'entry';
        $item['url'] = $post->getLink();
        $item['name'] = $post->getTitle();
        $item['content'] = (object) ['html' => $post->getDescription()];
        $item['author'] = (object) ['url' => $feed->getUrl(), 'name' => $feed->label()];
        $item['published'] = $this->dateFormatter->format($post->getPostedTime(), 'custom','Y-m-dTH:i:s');

        $items[] = (object) $item;
      }

      $pager = $this->pagerManager->getPager();
      $pager_total = $pager->getTotalPages();
      if (isset($pager_total) && is_numeric($pager_total)) {
        $page++;
        if ($pager_total > $page) {
          $paging = ['paging' => (object) ['after' => $page]];
        }
      }
    }

    return ['items' => $items] + $paging;
  }

}
